@@ -191,6 +191,9 @@ LockStruct  LockProcs[] =
 	{"forest",
 	 400000, 100, 200, 1, 64, 1.0, "",
 	 "Shows binary trees of a fractal forest", 0, NULL, NULL},
+	{"fzort",
+	 10000, 1, 1, 1, 64, 1.0, "",
+	 "Shows a metalic-looking fzort", 0, NULL, NULL},
 	{"galaxy",
 	 100, -5, 250, -3, 64, 1.0, "",
 	 "Shows crashing spiral galaxies", 0, NULL, NULL},
