@@ -2,10 +2,10 @@
 /* atunnels --- OpenGL Advanced Tunnel Screensaver */
 
 #if !defined( lint ) && !defined( SABER )
-/* static const char sccsid[] = "@(#)tunnel_draw.h    5.06 2002/09/20 xlockmore"; */
+/* static const char sccsid[] = "@(#)tunnel_draw.h    5.13 2004/07/19 xlockmore"; */
 #endif
 
-/* Copyright (c) E. Lassauge, 2002. */
+/* Copyright (c) E. Lassauge, 2002-2004. */
 
 /*
  * Permission to use, copy, modify, and distribute this software and its
@@ -24,7 +24,7 @@
  * Email: romka@ut.ee
  * WEB: http://romka.demonews.com
  *
- * Eric Lassauge  (March-16-2002) <lassauge@mail.dotcom.fr>
+ * Eric Lassauge  (March-16-2002) <lassauge AT users.sourceforge.net>
  *                                  http://lassauge.free.fr/linux.html
  *
  */
@@ -32,7 +32,8 @@
 #include <GL/gl.h>
 #include <GL/glx.h>
 
+#define MAX_TEXTURE	6
+
 extern void DrawTunnel(int do_texture, int do_light, GLuint *textures);
 extern void SplashScreen(int do_wire, int do_texture, int do_light);
 extern Bool InitTunnel(void);
-extern void FreeTunnel(void);
