@@ -408,7 +408,7 @@ init_goop(ModeInfo * mi)
 	int         i;
 	XGCValues   gcv;
 	int         nblobs;
-	unsigned long *plane_masks = 0;
+	unsigned long *plane_masks = NULL;
 	unsigned long base_pixel = 0;
 	goopstruct *gp;
 
@@ -487,7 +487,7 @@ init_goop(ModeInfo * mi)
 			gp->layers[i].pixel = base_pixel | plane_masks[i];
 		gp->background = base_pixel;
 	}
-	if (plane_masks)
+	if (plane_masks != NULL)
 		free(plane_masks);
 
 	if (gp->mode != transparent) {
