@@ -73,8 +73,6 @@ static const char sccsid[] = "@(#)toneclock.c	5.00 2004/09/16 xlockmore";
 #define DEF_MOVE "True"
 #define DEF_FILL "0"
 
-#define min(a,b) ((a) <= (b) ? (a) : (b))
-
 static Bool cycle_p , original , pulsating , move_clock;
 static int fill;
 
@@ -179,7 +177,7 @@ toneclock_drawhour(ModeInfo * mi, toneclockhour * hour0 ,
 #ifdef NO_DBUF
 	Window drawable = MI_WINDOW(mi);
 #else
-#define drawable tclock->dbuf
+#define drawable (Drawable) tclock->dbuf
 #endif
 	tclock = &toneclocks[MI_SCREEN(mi)];
 
@@ -485,13 +483,13 @@ init_toneclock(ModeInfo * mi)
 
 		if (tclock->dbuf_gc != None)
 			XFreeGC(display, tclock->dbuf_gc);
-		if ((tclock->dbuf_gc = XCreateGC(display, tclock->dbuf,
+		if ((tclock->dbuf_gc = XCreateGC(display, (Drawable) tclock->dbuf,
 			GCForeground | GCBackground | GCGraphicsExposures | GCFunction,
 				&gcv)) == None) {
 			XFreePixmap(display, tclock->dbuf);
 			tclock->dbuf = None;
 		} else {
-			XFillRectangle(display, tclock->dbuf, tclock->dbuf_gc,
+			XFillRectangle(display, (Drawable) tclock->dbuf, tclock->dbuf_gc,
 				0, 0, tclock->win_width, tclock->win_height);
 			XSetBackground(display, MI_GC(mi), MI_BLACK_PIXEL(mi));
 			XSetFunction(display, MI_GC(mi), GXcopy);
@@ -500,7 +498,7 @@ init_toneclock(ModeInfo * mi)
 #endif
 	tclock->angle = NRAND(360) * PI_RAD;
 	tclock->velocity = (NRAND(7) - 3) * PI_RAD;
-	size_hour = min( tclock->win_width , tclock->win_height) / 3;
+	size_hour = MIN( tclock->win_width , tclock->win_height) / 3;
    tclock->pulsating = False;
    tclock->moving = False;
    tclock->anglex = 0.0;
@@ -639,7 +637,7 @@ init_toneclock(ModeInfo * mi)
    tclock->a_y = 0;
    if ( !tclock->original && tclock->win_width > 20 )
      {
-	if ( tclock->radius < min( tclock->win_width , tclock->win_height) /
+	if ( tclock->radius < MIN( tclock->win_width , tclock->win_height) /
 	     4 )
 	  {
 	     if ( MI_IS_FULLRANDOM( mi ) )
@@ -690,7 +688,7 @@ draw_toneclock(ModeInfo * mi)
 	XSetFunction(display, tclock->gc, GXxor);
 #ifndef NO_DBUF
         XSetForeground(display,tclock->dbuf_gc,MI_BLACK_PIXEL(mi));
-	XFillRectangle(display, tclock->dbuf, tclock->dbuf_gc,
+	XFillRectangle(display, (Drawable) tclock->dbuf, tclock->dbuf_gc,
 		0, 0, tclock->win_width, tclock->win_height);
 #endif
 	tclock->painted = True;
@@ -756,11 +754,11 @@ draw_toneclock(ModeInfo * mi)
 	}
 #else
      XFlush(display);
-     XCopyArea (display, tclock->dbuf, window, tclock->dbuf_gc, 0, 0,
+     XCopyArea (display, (Drawable) tclock->dbuf, window, tclock->dbuf_gc, 0, 0,
 		     tclock->win_width, tclock->win_height, 0, 0);
      XFlush(display);
      XSetForeground(display,tclock->dbuf_gc,MI_BLACK_PIXEL(mi));
-     XFillRectangle(display, tclock->dbuf, tclock->dbuf_gc,
+     XFillRectangle(display, (Drawable) tclock->dbuf, tclock->dbuf_gc,
 		0, 0, tclock->win_width, tclock->win_height);
 #endif
 	XSetFunction(display, tclock->gc, GXcopy);
