@@ -20,6 +20,11 @@ typedef struct {
 	unsigned long	foreground;
 } TColor;
 
+typedef struct {
+	int	hasFillRule;
+	int	value;
+} TFillRule;
+
 typedef struct {
 	int	hasFillStyle;
 	int	value;
@@ -48,13 +53,21 @@ typedef struct {
 	Pixmap	graphic;
 } TStipple;
 
+typedef struct {
+	int hasOrigin;
+	int	x;
+	int y;
+} TTSOrigin;
+
 typedef struct {
 	int				isActive;
 	int				needsReset;
 	TColor			color;
+	TFillRule		fillRule;
 	TFillStyle		fillStyle;
 	TFunction		function;
 	TLineAttributes	lineAttributes;
+	TTSOrigin		tsorigin;
 	TStipple		stipple;
 	TFont           font;
 } TGCInfo;
@@ -62,6 +75,8 @@ typedef struct {
 typedef struct {
 	int		isActive;
 	int		type;
+	int		width;
+	int		height;
 	HBITMAP	bitmap;
 } TBMInfo;
 
@@ -288,6 +303,8 @@ int GCCreate(void)
 	gcInfo[retVal].needsReset = TRUE;
 	gcInfo[retVal].color.background = NUMCOLORS+1;
 	gcInfo[retVal].color.foreground = NUMCOLORS;
+	gcInfo[retVal].fillRule.hasFillRule = FALSE;
+	gcInfo[retVal].fillRule.value = 0;
 	gcInfo[retVal].fillStyle.hasFillStyle = FALSE;
 	gcInfo[retVal].fillStyle.value = 0;
 	gcInfo[retVal].function.hasFunction = FALSE;
@@ -297,6 +314,9 @@ int GCCreate(void)
 	gcInfo[retVal].lineAttributes.lineStyle = 0;
 	gcInfo[retVal].lineAttributes.capStyle = 0;
 	gcInfo[retVal].lineAttributes.joinStyle = 0;
+	gcInfo[retVal].tsorigin.hasOrigin = FALSE;
+	gcInfo[retVal].tsorigin.x = 0;
+	gcInfo[retVal].tsorigin.y = 0;
 	gcInfo[retVal].stipple.hasStipple = FALSE;
 	gcInfo[retVal].stipple.graphic = -1;
 
@@ -308,6 +328,7 @@ HDC GCGetDC(Drawable d, GC gc)
 {
 	HGDIOBJ			brush;
 	int				fillfunc;
+	int				fillrule;
 	int				i;
 	LOGBRUSH		lb;
 	LPLOGPALETTE	lpLogPal;
@@ -513,6 +534,18 @@ HDC GCGetDC(Drawable d, GC gc)
 	DeleteObject(palette);
 	LocalFree(lpLogPal);
 
+	/* set brush origin */
+	if (gcInfo[gc].tsorigin.hasOrigin == TRUE)
+	{
+		if (SetBrushOrgEx(lhdc, gcInfo[gc].tsorigin.x,
+				gcInfo[gc].tsorigin.y, NULL) == 0)
+		{
+			sprintf(message, "brush origin: SetBrushOrgEx() failed");
+			xlockmore_set_debug(message);
+			return NULL;
+		}
+	}
+
     /* check if we need a pattern brush or solid brush */
 	if (gcInfo[gc].stipple.hasStipple == FALSE)
 	{
@@ -521,7 +554,12 @@ HDC GCGetDC(Drawable d, GC gc)
 	        CreateSolidBrush(RGB(cred, cgreen, cblue)));
 	    DeleteObject(brush);
 	}
-	else if (gcInfo[gc].stipple.graphic != -1)
+	/* check if we need to FillStippled, and if so don't make a
+	 * brush, we will deal with this later (see XFillRectangle)
+	 */
+	else if (gcInfo[gc].stipple.graphic != -1 && 
+		! (gcInfo[gc].fillStyle.hasFillStyle == TRUE && 
+		   gcInfo[gc].fillStyle.value == FillStippled))
 	{
 		/* pattern brush */
   		HBRUSH oldBrush;
@@ -531,7 +569,11 @@ HDC GCGetDC(Drawable d, GC gc)
   		RGBQUAD rgbQuad = { cblue, cgreen, cred, 0 };
 
 		if ((stipple = DrawableGetBitmap(gcInfo[gc].stipple.graphic)) == NULL)
+		{
+			sprintf(message, "pattern brush: DrawableGetBitmap() returned NULL");
+			xlockmore_set_debug(message);
 			return NULL;
+		}
 
   		/* create a device context for setting the stipples color */
   		if ((hStippleDC = CreateCompatibleDC(lhdc)) == NULL)
@@ -612,6 +654,23 @@ HDC GCGetDC(Drawable d, GC gc)
   		SetROP2(lhdc, fillfunc);
 	}
 
+	/* set fill rule */
+	if (gcInfo[gc].fillRule.hasFillRule == TRUE)
+	{
+		switch (gcInfo[gc].fillRule.value)
+		{
+			case EvenOddRule:	fillrule = ALTERNATE; break;
+			case WindingRule:	fillrule = WINDING; break;
+			default:			fillrule = WINDING; break;
+		}
+		if (SetPolyFillMode(lhdc, fillrule) == 0)
+		{
+			sprintf(message, "fill rule: cannot set fill mode in DC");
+			xlockmore_set_debug(message);
+			return NULL;
+		}
+	}
+
 	/* 
 	 * if we call this function again and there is no change to the DC
 	 * that needs to be done, we want to just return the DC that had
@@ -651,6 +710,8 @@ int BMInfoCreate(void)
 
 	/* set the bitmap info */
 	bmInfo[retVal].isActive = TRUE;
+	bmInfo[retVal].width = 0;
+	bmInfo[retVal].height = 0;
 	bmInfo[retVal].bitmap = NULL;
 
 	return retVal;
@@ -904,6 +965,10 @@ void XChangeGC(Display *display, GC gc, unsigned long valuemask,
 	{
 		XSetBackground(display, gc, values->background);
 	}
+	if (valuemask & GCFillRule)
+	{
+		XSetFillRule(display, gc, values->fill_rule);
+	}
 	if (valuemask & GCFillStyle)
 	{
 		XSetFillStyle(display, gc, values->fill_style);
@@ -956,7 +1021,15 @@ void XChangeGC(Display *display, GC gc, unsigned long valuemask,
 	{
 		XSetFont(display, gc, values->font);
 	}
-	
+	if (valuemask & GCTileStipXOrigin)
+	{
+		XSetTSOrigin(display, gc, values->ts_x_origin, gcInfo[gc].tsorigin.y);
+	}
+	if (valuemask & GCTileStipYOrigin)
+	{
+		XSetTSOrigin(display, gc, gcInfo[gc].tsorigin.x, values->ts_y_origin);
+	}
+
 	/* the above are the only attributes we can set at this stage */
 }
 
@@ -1024,31 +1097,50 @@ int XCopyArea(Display *display, Drawable src, Drawable dest, GC gc,
 			  int dest_x, int dest_y)
 {
 	HBITMAP	hBitmap;
-	HBITMAP	hOldBitmap;
+	HBITMAP	hOldBitmapS;
+	HBITMAP	hOldBitmapD;
 	HDC		hDestDC;
 	HDC		hSrcDC;
 
+	/* create the destination resource */
 	if ((hDestDC = GCGetDC(dest, gc)) == NULL)
 	{
-		xlockmore_set_debug("XCopyArea: Failed to get dc");
+		xlockmore_set_debug("XCopyArea: Failed to get destination DC");
 		return BadGC;
 	}
 
+	if (dest > 0 && dest < NUMBER_BITMAP)
+		if (bmInfo[dest].isActive == TRUE)
+			hOldBitmapD = SelectObject(hDestDC, DrawableGetBitmap(dest));
+
 	/* create the source resource */
-	if ((hBitmap = DrawableGetBitmap(src)) == NULL)
+	if (src > 0 && src <= numBMInfo)
 	{
-		xlockmore_set_debug("XCopyArea: Failed to get bitmap");
-		return BadDrawable;
-	}
+		/* source is a bitmap */
+		if ((hBitmap = DrawableGetBitmap(src)) == NULL)
+		{
+			xlockmore_set_debug("XCopyArea: Failed to get bitmap");
+			return BadDrawable;
+		}
 
-	if ((hSrcDC = CreateCompatibleDC(hdc)) == NULL)
+		if ((hSrcDC = CreateCompatibleDC(hdc)) == NULL)
+		{
+			xlockmore_set_debug("XCopyArea: Failed to create compatible DC");
+			return BadDrawable;
+		}
+
+		hOldBitmapS = SelectObject(hSrcDC, hBitmap);
+	}
+	else
 	{
-		xlockmore_set_debug("XCopyArea: Failed to create compatible DC");
-		return BadDrawable;
+		/* source is a window */
+		if ((hSrcDC = GetDC((HWND)src)) == NULL)
+		{
+			xlockmore_set_debug("XCopyArea: Failed to get source DC");
+			return BadDrawable;
+		}
 	}
 
-	hOldBitmap = SelectObject(hSrcDC, hBitmap);
-
 	/* copy src to destination */
 	if (BitBlt(hDestDC, dest_x, dest_y, width, height, hSrcDC, 
 		src_x, src_y, SRCCOPY) == 0)
@@ -1057,9 +1149,14 @@ int XCopyArea(Display *display, Drawable src, Drawable dest, GC gc,
 	}
 
 	/* clean up */
-	if (hOldBitmap)
-		SelectObject(hSrcDC, hOldBitmap);
-	DeleteDC(hSrcDC);
+	if (hOldBitmapS)
+		SelectObject(hSrcDC, hOldBitmapS);
+	if (hOldBitmapD)
+		SelectObject(hSrcDC, hOldBitmapD);
+	if (src > 0 && src <= numBMInfo)
+		DeleteDC(hSrcDC);
+	else
+		ReleaseDC((HWND)src, hSrcDC);
 	return Success;
 }
 
@@ -1072,6 +1169,17 @@ Colormap XCopyColormapAndFree(Display *display, Colormap colormap)
 	return 0;
 }
 
+/*-
+ *  XCopyPlane
+ *    to be implemented
+ */
+int XCopyPlane(Display *display, Drawable src, Drawable dest, GC gc,
+			  int src_x, int src_y, unsigned width, int height, 
+			  int dest_x, int dest_y, unsigned long plane)
+{
+	return 0;
+}
+
 /*-
  *  XCreateBitmapFromData
  *    used to create a bitmap
@@ -1104,6 +1212,8 @@ Pixmap XCreateBitmapFromData(Display *display, Drawable drawable,
 
 	bmInfo[pm].bitmap = hBitmap;
 	bmInfo[pm].type = 1;
+	bmInfo[pm].width = width;
+	bmInfo[pm].height = height;
 	return pm;
 }
 
@@ -1201,6 +1311,8 @@ Pixmap XCreatePixmap(Display *display, Drawable d, unsigned int width,
 
 	bmInfo[pm].bitmap = hBitmap;
 	bmInfo[pm].type = 0;
+	bmInfo[pm].width = width;
+	bmInfo[pm].height = height;
 
 	return pm;
 }
@@ -1219,19 +1331,52 @@ Cursor XCreatePixmapCursor(Display *display,
 
 /*-
  *  XCreatePixmapFromBitmapData
- *    not currently used, returns 0
+ *    Creates a pixmap from bitmap data
  */
 Pixmap XCreatePixmapFromBitmapData(Display *display, Drawable drawable,
 	    char *data, unsigned int width, unsigned int height,
 		unsigned long fg, unsigned long bg, unsigned int depth)
 {
-	Pixmap pm;
+	HDC		hBitmapDC;
+	HBITMAP	hOldBitmap;
+	Pixmap	pm;
+	RGBQUAD	rgbQuad[2] = { 0 };
 
+	/* create the bitmap */
+	if ((pm = XCreateBitmapFromData(display, drawable, data, width, height)) == None)
+	{
+		return None;
+	}
+
+	/* if depth is 1 create the bitmap using black and white */
 	if (depth == 1)
-		if ((pm = XCreateBitmapFromData(display, drawable, data, width, height)) != None)
-			return pm;
-	
-	return None;
+	{
+		fg = 257;
+		bg = 256;
+	}
+
+	/* set the correct colors in the color table */
+	rgbQuad[0].rgbRed = red[fg];
+	rgbQuad[0].rgbGreen = green[fg];
+	rgbQuad[0].rgbBlue = blue[fg];
+	rgbQuad[0].rgbReserved = 0;
+	rgbQuad[1].rgbRed = red[bg];
+	rgbQuad[1].rgbGreen = green[bg];
+	rgbQuad[1].rgbBlue = blue[bg];
+	rgbQuad[1].rgbReserved = 0;
+
+	/* create a DC so we can set the bitmaps colortable */
+  	hBitmapDC = CreateCompatibleDC(hdc);
+	hOldBitmap = SelectObject(hBitmapDC, DrawableGetBitmap(pm));
+
+	/* set the colortable */
+	SetDIBColorTable(hBitmapDC, 0, 2, rgbQuad);
+
+	/* clean up */
+	SelectObject(hBitmapDC, hOldBitmap);
+	DeleteDC(hBitmapDC);
+
+	return pm;
 }
 
 /*
@@ -1797,22 +1942,73 @@ void XFillRectangle(Display *display, Drawable d, GC gc, int x, int y,
 		if (bmInfo[d].isActive == TRUE)
 			hOldBitmap = SelectObject(dc, DrawableGetBitmap(d));
 
-	if (width == 1 && height == 1)
+    /* check if we are doing a FillStippled fill or a normal rectangle */
+	if (gcInfo[gc].fillStyle.hasFillStyle == TRUE && 
+		gcInfo[gc].fillStyle.value == FillStippled)
 	{
-		SetPixelV(dc, x, y, RGB(cred, cgreen, cblue));
+		HBITMAP hOldBitmapS = NULL;
+		HBITMAP hStipple = NULL;
+		HDC hStippleDC;
+		int i, j;
+  		RGBQUAD rgbQuad = { cblue, cgreen, cred, 0 };
+		int sHeight;
+		int sWidth;
+
+		/* FillStippled fill */
+		if ((hStipple = DrawableGetBitmap(gcInfo[gc].stipple.graphic)) == NULL)
+		{
+			xlockmore_set_debug("XFillRectangle: Failed to get stipple");
+			if (hOldBitmap)
+				SelectObject(dc, hOldBitmap);
+			return;
+		}
+		sWidth = bmInfo[gcInfo[gc].stipple.graphic].width;
+		sHeight = bmInfo[gcInfo[gc].stipple.graphic].height;
+		if ((hStippleDC = CreateCompatibleDC(hdc)) == NULL)
+		{
+			xlockmore_set_debug("XFillRectangle: Failed to create stipple DC");
+			if (hOldBitmap)
+				SelectObject(dc, hOldBitmap);
+			return;
+		}
+		hOldBitmapS = SelectObject(hStippleDC, hStipple);
+
+		/* set the correct color table */
+		SetDIBColorTable(hStippleDC, 0, 1, &rgbQuad);
+
+		for (i=y; i<y+height; i+=sHeight)
+			for (j=x; j<x+width; j+=sWidth)
+				if (MaskBlt(dc, j, i, sWidth, sHeight, hStippleDC, 0, 0, hStipple, 0, 0, 
+						MAKEROP4(0x00AA0029, SRCCOPY)) == 0)
+					xlockmore_set_debug("XFillRectangle: MaskBlt() failed");
+
+		/* clean up */
+		if (hOldBitmapS)
+			SelectObject(hStippleDC, hOldBitmapS);
+		DeleteDC(hStippleDC);
 	}
 	else
 	{
-		if (noPen == NULL)
-			if ((noPen = CreatePen(PS_NULL, 0, RGB(0,0,0))) == NULL)
-			{
-				xlockmore_set_debug("Failed to create pen");
-				return;
-			}
+		/* normal rectangle */
+		if (width == 1 && height == 1)
+		{
+			SetPixelV(dc, x, y, RGB(cred, cgreen, cblue));
+		}
+		else
+		{
+			if (noPen == NULL)
+				if ((noPen = CreatePen(PS_NULL, 0, RGB(0,0,0))) == NULL)
+				{
+					xlockmore_set_debug("XFillRectangle: Failed to create pen");
+					if (hOldBitmap)
+						SelectObject(dc, hOldBitmap);
+					return;
+				}
 
-		oldPen = SelectObject(dc, noPen);
-		Rectangle(dc, x, y, x + width + 1, y + height + 1);
-		SelectObject(dc, oldPen);
+			oldPen = SelectObject(dc, noPen);
+			Rectangle(dc, x, y, x + width + 1, y + height + 1);
+			SelectObject(dc, oldPen);
+		}
 	}
 
 	if (hOldBitmap)
@@ -1921,9 +2117,21 @@ void XFreePixmap(Display *display, Pixmap pixmap)
 		DeleteObject(bmInfo[pixmap].bitmap);
 		bmInfo[pixmap].isActive = FALSE;
 		bmInfo[pixmap].bitmap = NULL;
+		bmInfo[pixmap].width = 0;
+		bmInfo[pixmap].height = 0;
 	}
 }
 
+/*-
+ *  XGetGCValues
+ *    not currently used
+ */
+Status XGetGCValues(Display* display, GC gc,
+					unsigned long valuemask, XGCValues* values_return)
+{
+    return None;
+}
+
 /*-
  *  XGContextFromGC
  *    not currently used
@@ -2149,6 +2357,16 @@ void XQueryColor(Display *display, Colormap colormap, XColor *def_in_out)
 {
 }
 
+/*-
+ *  XQueryFont
+ *    not currently used
+ */
+XFontStruct *XQueryFont(Display* display, XID font_ID)
+{
+	static XFontStruct dummy;
+	return &dummy;
+}
+
 /*-
  *  XQueryPointer
  *    not currently used
@@ -2162,17 +2380,16 @@ Bool XQueryPointer(Display *display, Window w, Window *root_return,
 }
 
 /*-
- *  XQueryFont
+ *  XQueryTree
  *    not currently used
  */
-XFontStruct *XQueryFont(Display* display, XID font_ID)
+Status XQueryTree(Display *display, Window w, Window *root_return, 
+				 Window *parent_return, Window **children_return,
+				 unsigned int *nchildren_return)
 {
-	static XFontStruct dummy;
-	return &dummy;
+	return 0;
 }
 
-
-
 /*-
  *  XRaiseWindow
  *    not currently used
@@ -2312,6 +2529,20 @@ void XSetBackground(Display *display, GC gc, unsigned long background)
 	}
 }
 
+/*-
+ *  XSetFillRule
+ *    sets the fill rule
+ */
+void XSetFillRule(Display *display, GC gc, int fill_rule)
+{
+	if (gc >= 0 && gc < numGCInfo && gcInfo[gc].isActive == TRUE)
+	{
+		gcInfo[gc].needsReset = TRUE;
+		gcInfo[gc].fillRule.hasFillRule = TRUE;
+		gcInfo[gc].fillRule.value = fill_rule;
+	}
+}
+
 /*-
  *  XSetFillStyle
  *    sets the fill style
@@ -2429,11 +2660,17 @@ void XSetStipple(Display *display, GC gc, Pixmap stipple)
 
 /*-
  *  XSetTSOrigin
- *  Not implemented yet.
+ *     Set the tile or stipple origin
  */
-
 void XSetTSOrigin(Display *display, GC gc, int ts_x_origin, int ts_y_origin)
 {
+	if (gc >= 0 && gc < numGCInfo && gcInfo[gc].isActive == TRUE)
+	{
+		gcInfo[gc].needsReset = TRUE;
+		gcInfo[gc].tsorigin.hasOrigin = TRUE;
+		gcInfo[gc].tsorigin.x = ts_x_origin;
+		gcInfo[gc].tsorigin.y = ts_y_origin;
+	}
 }
 
 /*-
