@@ -70,6 +70,14 @@ extern RECT rc;                           /* coords of the screen */
 #define BadLength         16    /* Request length incorrect */
 #define BadImplementation 17    /* server is defective */
 
+/* window attributes */
+#define InputOutput        1
+#define InputOnly          2
+
+#define IsUnmapped         0
+#define IsUnviewable       1
+#define IsViewable         2
+
 /* redefined functions */
 #define XWhitePixelOfScreen WhitePixelOfScreen
 
@@ -117,6 +125,14 @@ extern RECT rc;                           /* coords of the screen */
 #define Mod4Mask                (1<<6)
 #define Mod5Mask                (1<<7)
 
+/* button masks.  Used in same manner as Key masks above. Not to be confused
+   with button names below. */
+#define Button1Mask             (1<<8)
+#define Button2Mask             (1<<9)
+#define Button3Mask             (1<<10)
+#define Button4Mask             (1<<11)
+#define Button5Mask             (1<<12)
+
 
 
 #define VisualNoMask           (0x0)
@@ -165,7 +181,7 @@ extern RECT rc;                           /* coords of the screen */
 #define FocusChangeMask                 (1L<<21) 
 #define PropertyChangeMask              (1L<<22) 
 #define ColormapChangeMask              (1L<<23) 
-#define OwnerGrabButtonMask             (1L<<24) 
+#define OwnerGrabButtonMask             (1L<<24)
 
 #define KeyPress                2
 #define KeyRelease              3
@@ -239,7 +255,6 @@ extern RECT rc;                           /* coords of the screen */
 #define JoinBevel (2)
 
 
-/*
 #define   GCFunction                  (1L<<0)
 #define   GCPlaneMask                 (1L<<1)
 #define   GCForeground                (1L<<2)
@@ -263,19 +278,6 @@ extern RECT rc;                           /* coords of the screen */
 #define   GCDashOffset                (1L<<20)
 #define   GCDashList                  (1L<<21)
 #define   GCArcMode                   (1L<<22)
-*/
-
-#define   GCFunction                  (1L<<0)
-#define   GCForeground                (1L<<2)
-#define   GCBackground                (1L<<3)
-#define   GCLineWidth                 (1L<<4)
-#define   GCLineStyle                 (1L<<5)
-#define   GCCapStyle                  (1L<<6)
-#define   GCJoinStyle                 (1L<<7)
-#define   GCFillStyle                 (1L<<8)
-#define   GCStipple                   (1L<<11)
-#define   GCFont                      (1L<<14)
-#define   GCGraphicsExposures         (1L<<16)
 
 #define FillSolid          (0)
 #define FillTiled          (1)
@@ -299,6 +301,10 @@ extern RECT rc;                           /* coords of the screen */
 #define GXnand			(0xe)
 #define GXset			(0xf)
 
+/* fillRule */
+#define EvenOddRule             0
+#define WindingRule             1
+
 /*-
  * types
  */
@@ -343,6 +349,9 @@ typedef XID GContext;
 /* Keysym type: not needed for WIN32 */
 typedef XID KeySym;
 
+/* Pixel type, from <X11/Intrinsic.h> */
+typedef unsigned long Pixel;
+
 /* Pixmap type, implemented a bitmap handler for WIN32 */
 typedef int Pixmap;
 
@@ -798,6 +807,9 @@ void XConfigureWindow(Display *display, Window w,
 int XCopyArea(Display *display, Drawable src, Drawable dest, GC gc, 
               int src_x, int src_y, unsigned int width, unsigned height,
 			  int dest_x, int dest_y);
+int XCopyPlane(Display *display, Drawable src, Drawable dest, GC gc,
+              int src_x, int src_y, unsigned width, int height, int dest_x,
+              int dest_y, unsigned long plane);
 Colormap XCopyColormapAndFree(Display *display, Colormap colormap);	
 Pixmap XCreateBitmapFromData(Display *display, Drawable drawable, 
                              char *data, unsigned int width,
@@ -897,11 +909,14 @@ void XPutImage(Display *display, Drawable d, GC gc, XImage *image, int src_x,
                int src_y, int dest_x, int dest_y, unsigned int width,
                unsigned int height);
 void XQueryColor(Display *display, Colormap colormap, XColor *def_in_out);
+XFontStruct *XQueryFont(Display* display, XID font_ID);
 Bool XQueryPointer(Display *display, Window w, Window *root_return,
 				   Window *child_return, int *root_x_return, int *root_y_return,
 				   int *win_x_return, int *win_y_return,
 				   unsigned int *mask_return);
-XFontStruct *XQueryFont(Display* display, XID font_ID);
+Status XQueryTree(Display *display, Window w, Window *root_return, 
+                 Window *parent_return, Window **children_return,
+                 unsigned int *nchildren_return);
 void XRaiseWindow(Display *display, Window w);
 int XReadBitmapFile(Display *display, Drawable d, char *filename,
                     unsigned int *width_return, unsigned int *height_return,
@@ -920,6 +935,7 @@ void XrmParseCommand(XrmDatabase *database, XrmOptionDescList table,
 					 int table_count, char *name, int *argc_in_out,
 					 char **argv_in_out);
 void XSetBackground(Display *display, GC gc, unsigned long background);
+void XSetFillRule(Display *display, GC gc, int fill_rule);
 void XSetFillStyle(Display *display, GC gc, int fill_style);
 void XSetFont(Display *display, GC gc, Font font);
 void XSetForeground(Display *display, GC gc, unsigned long foreground);
