@@ -267,6 +267,12 @@ LockStruct  LockProcs[] =
 	 400000, 100, 200, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows binary trees of a fractal forest", 0, NULL},
 #endif
+#ifdef MODE_fzort
+	{(char *) "fzort", init_fzort, draw_fzort, release_fzort,
+	 refresh_fzort, init_fzort, (ModeHook *) NULL, &fzort_opts,
+	 10000, 1, 1000, 1, 64, 1.0, (char *) "",
+	 (char *) "Draws a metallic-looking fzort.", 0, NULL},
+#endif
 #ifdef MODE_galaxy
 	{(char *) "galaxy", init_galaxy, draw_galaxy, release_galaxy,
 	 refresh_galaxy, init_galaxy, (ModeHook *) NULL, &galaxy_opts,
@@ -824,12 +830,10 @@ LockStruct  LockProcs[] =
 	 1, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a random mode (except blank and bomb)", 0, NULL},
 #else
-#ifndef WIN32
 	{(char *) "random", init_random, draw_random, release_random,
 	 refresh_random, change_random, (ModeHook *) NULL, &random_opts,
 	 1, 1, 1, 1, 64, 1.0, (char *) "",
 	 (char *) "Shows a random mode (except blank)", 0, NULL},
-#endif /* !WIN32 */
 #endif
 };
 
