@@ -544,14 +544,10 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_ant3d
 #define MODE_apollonian
 #define MODE_ball
-#ifndef WIN32
 #define MODE_bat
-#endif /* !WIN32 */
 #define MODE_blot
 #define MODE_bouboule
-#ifndef WIN32
 #define MODE_bounce
-#endif /* !WIN32 */
 #define MODE_braid
 #ifndef WIN32
 #define MODE_bubble
@@ -574,16 +570,12 @@ extern void release_last_mode(ModeInfo *);
 #endif /* !WIN32 */
 #define MODE_deco
 #define MODE_demon
-#ifndef WIN32
 #define MODE_dilemma
-#endif /* !WIN32 */
 #define MODE_discrete
 #define MODE_dragon
 #define MODE_drift
 #define MODE_euler2d
-#ifndef WIN32
 #define MODE_eyes
-#endif /* !WIN32 */
 #define MODE_fadeplot
 #define MODE_fiberlamp
 #ifndef WIN32
@@ -592,6 +584,9 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_flame
 #define MODE_flow
 #define MODE_forest
+#ifndef WIN32
+#define MODE_fzort
+#endif /* !WIN32 */
 #define MODE_galaxy
 #ifndef WIN32
 #define MODE_goop
@@ -603,14 +598,12 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_ico
 #ifndef WIN32
 #define MODE_ifs
-#define MODE_image
 #endif /* !WIN32 */
+#define MODE_image
 #define MODE_juggle
 #define MODE_julia
 #define MODE_kaleid
-#ifndef WIN32
 #define MODE_kumppa
-#endif /* !WIN32 */
 #define MODE_laser
 #define MODE_life
 #define MODE_life1d
@@ -623,14 +616,14 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_mandelbrot
 #ifndef WIN32
 #define MODE_marquee
-#define MODE_matrix
 #endif /* !WIN32 */
+#define MODE_matrix
 #define MODE_maze
 #define MODE_mountain
 #define MODE_munch
-#ifndef WIN32
 #define MODE_nose
 #define MODE_pacman
+#ifndef WIN32
 #define MODE_penrose
 #endif /* !WIN32 */
 #define MODE_petal
@@ -644,20 +637,16 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_roll
 #define MODE_rotor
 #define MODE_scooter /* should be combined with star some day */
-#ifndef WIN32
 #define MODE_shape
-#endif /* !WIN32 */
 #define MODE_sierpinski
-#ifndef WIN32
 #define MODE_slip
-#endif /* !WIN32 */
 #define MODE_space /* should be combined with star some day */
 #define MODE_sphere
 #define MODE_spiral
 #define MODE_spline
-#ifndef WIN32
 #define MODE_star
 #define MODE_starfish
+#ifndef WIN32
 #define MODE_strange
 #endif /* !WIN32 */
 #define MODE_swarm
@@ -674,13 +663,9 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_turtle
 #define MODE_vines
 #define MODE_voters
-#ifndef WIN32
 #define MODE_wator
-#endif /* !WIN32 */
 #define MODE_wire
-#ifndef WIN32
 #define MODE_world
-#endif /* !WIN32 */
 #define MODE_worm
 #ifndef WIN32
 #define MODE_xcl
@@ -711,6 +696,7 @@ extern void release_last_mode(ModeInfo *);
 #define MODE_fadeplot
 #define MODE_flame
 #define MODE_forest
+#define MODE_fzort
 #define MODE_grav
 #define MODE_hop
 #define MODE_hyper
@@ -1079,6 +1065,14 @@ extern ModeHook refresh_forest;
 extern ModeSpecOpt forest_opts;
 #endif
 
+#ifdef MODE_fzort
+extern ModeHook init_fzort;
+extern ModeHook draw_fzort;
+extern ModeHook release_fzort;
+extern ModeHook refresh_fzort;
+extern ModeSpecOpt fzort_opts;
+#endif
+
 #ifdef MODE_galaxy
 extern ModeHook init_galaxy;
 extern ModeHook draw_galaxy;
